/* d_array.c
 *
 *  Copyright (C) 2006,2007 Tuomo Makinen (tjam@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "d_array.h"

/* Initializes d_array structure */
struct d_array *
d_array_create ()
{
  struct d_array *array;

  /* Allocate array */
  array = (struct d_array *) malloc (sizeof (struct d_array));
  if (array == NULL)
    return NULL;

  array->__size = 0;

  return array;
}

int
d_array_delete (struct d_array *array)
{
  int i;

  for (i = 0; i < array->__size; i++)
    {
      if (array->element[i]->payload != NULL)
        free (array->element[i]->payload);
      free (array->element[i]);
    }
  if (array->__size > 0 &&
      array->element != NULL)
    free (array->element);
  free (array);

  array = NULL;

  return 0;
}

unsigned int
d_array_get_size (struct d_array *array)
{
  return array->__size;
}

struct member *
d_array_get_element (struct d_array *array, unsigned int index)
{
  if (index < 0 || index > array->__size - 1)
    return NULL;

  return array->element[index];
}

int
d_array_add_element (struct d_array *array)
{
  struct member **newp;

  if (array->__size < 0)
    return -1;

  if (array->__size == 0)
    {
      /* Allocate array element pointer */
      array->element = (struct member **) malloc (sizeof (struct member *));
      if (array->element == NULL)
        return -2;
    }
  else
    {
      /* Grow member array */
      newp =
	(struct member **) realloc (array->element,
				    (array->__size + 1) *
				    sizeof (struct member *));
      if (newp == NULL)
        return -2;

      array->element = newp;
    }

  /* Allocate new element in run_action array */
  array->element[array->__size] =
    (struct member *) malloc (sizeof (struct member));
  if (array->element[array->__size] == NULL)
    return -2;

  /* Initialize member payload as NULL */
  array->element[array->__size]->payload = NULL;
  
  array->__size++;

  return array->__size - 1;
}
