/* nuhe_global.h
 *
 * Copyright (C) 2006,2007 Tuomo Makinen (tjam@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef NUHE_GLOBAL_H
#define NUHE_GLOBAL_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <time.h>
#include <pthread.h>

#ifdef HAVE_PCRE_PCRE_H
#include <pcre/pcre.h>
#else
#include <pcre.h>
#endif

#include "slist.h"
#include "d_array.h"

/* FMON_SLEEPTIME defines amount of idle time
 * for file monitoring (microseconds) between 
 * file reads, when read is unsuccesful
 */
#define FMON_SLEEPTIME 1000

/* Time to sleep between events */
#define EVENT_SLEEPTIME 1000

/* Pcre oveccount value */
#define OVECCOUNT 30

/* Maximum amount of substrings to be captured */
#define SUBSTRMAX 9

#define STRBUF 256

#define BUFSIZE 1024

#define LOGBUF 4096

/* action indentifiers */
enum actions
{
  EMPTY = 0,
  CMD
};

/* logging level for actions */
enum logging
{
  NONE = 0,
  INFO,
  ALERT,
  CRIT
};

/* Logging treshold */
enum logtreshold
{
  ALWAYS = 0,
  EVENT
};

/* Nuhe running status */
enum status
{
  STARTING = 0,
  DAEMONIC,
  FOREGROUD,
  SCANLOGS,
  FREEZE,
  DEFUNC
};

/* status for quit_flag */
enum
{
  RUNNING = 0,
  SHUTDOWN,
  QUIT,
  RESTART
};

/* Struct containing pthread mutex & cv for Nuhe states */
struct sync_state
{
  pthread_mutex_t state_mutex;
  pthread_cond_t state_cv;
};

/* Rule group structure */
struct rulegroup
{
  struct list_s *rglist;
};

/* Rule info structure */
struct rulefiles
{
  char *fpath;
  char *fname;
};

/* Rule structure */
struct rule
{
  struct action *aptr;
  struct rulegroup *group;

  unsigned int rulenr;
  char *actname;
  char *desc;

  pcre *re;
};

/* Structure for action handler */
struct do_action
{
  char *action;
  int event_max;
  time_t t_limit;

  /* Resetable flag for action phase */
  unsigned int allowrst;
};

/* Structure for action */
struct action
{
  char *name;
  unsigned int usesubstr[SUBSTRMAX];

  struct d_array *ignore;
  struct d_array *run_action;

  enum actions id;
  enum logging loglevel;
  enum logtreshold ltvalue;

  /* reset variables */
  char *resetcmd;
  int substrtable[SUBSTRMAX];
  struct d_array *reset;

  /* Flag to indicate if multiple alert
   * sources are used to update event
   */
  unsigned int multisource;
};

/* Main structure for Nuhe */
struct internal
{
  struct d_array *logs;
  struct d_array *rules;

  struct list_s *rlist;
  struct list_s *alist;
  struct list_s *elist;

  enum status run_stat;

  unsigned int managerport;
  unsigned int manageralrt;
  unsigned int active_fmon;
 
  char **argvptr;
  char manageraddr[STRBUF];
  char sensorname[STRBUF];
  char logfile[STRBUF];
  char pidfile[STRBUF];
  char ruledir[STRBUF];
  char eventfile[STRBUF];
  char privatekey[STRBUF];
  char certificate[STRBUF];
  char cafile[STRBUF];
  char cadir[STRBUF];
  char *cfile;
  char *errstr;

  pcre *lmsg;

  pthread_t event_t;
  pthread_t nodem_t;
};

/* Synchronize FREEZE state */
struct sync_state freeze_sync;

#endif	/* NUHE_GLOBAL_H */
