/* slist.c
 *
 *  Copyright (C) 2006,2007 Tuomo Makinen (tjam@users.sourceforge.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "slist.h"

static void slist_allocate_node (struct list_s **new_node);

void
slist_allocate_node (struct list_s **new_node)
{
  *new_node = (struct list_s *) malloc (sizeof (struct list_s));
}

struct list_s *
slist_push (struct list_s **head)
{
  struct list_s *new_node;

  slist_allocate_node (&new_node);

  if (new_node == NULL)
    return NULL;
    
  new_node->next = *head;
  *head = new_node;
  
  return new_node;
}

struct list_s *
slist_append_node (struct list_s **head)
{
  struct list_s *new_node;
  struct list_s *current;
  struct list_s *tail;

  current = *head;

  slist_allocate_node (&new_node);

  if (new_node == NULL)
    return NULL;

  if (current == NULL)
    {
      new_node->next = *head;
      *head = new_node;
    }
  else
    {
      while (current != NULL)
        {
          tail = current;
          current = current->next;
        }
      new_node->next = NULL;
      tail->next = new_node;
    }
  return new_node;
}

void
slist_delete_list (struct list_s **head)
{
  struct list_s *current;
  struct list_s *next;

  current = *head;

  while (current != NULL)
    {
      next = current->next;
      free (current->payload);
      free (current);

      current = next;
    }
  *head = NULL;
}
